@@ -34,7 +34,7 @@
 #define TITLE3_FORMAT "%s"
 #define TIME_FORMAT "%s"
 #define SCALE_WIDTH 150
-#define COVER_SIZE 80
+#define COVER_SIZE 100
 #define TRAY_COVER_SIZE 80
 #define MIN_WIDTH 500
 #define MIN_TOOLTIP_WIDTH 300
@@ -62,9 +62,9 @@ struct _GooPlayerInfoPrivateData {
 	gint64       track_length;
 	gboolean     dragging;
 	guint        update_id;
-
 	double       fraction;
 	guint        update_progress_timeout;
+	GdkPixbuf   *original_cover;
 };
 
 
@@ -90,8 +90,6 @@ goo_player_info_get_preferred_width (GtkWidget *widget,
 				     int       *minimum_width,
 				     int       *natural_width)
 {
-	GooPlayerInfo *info = GOO_PLAYER_INFO (widget);
-
 	*minimum_width = *natural_width = MIN_WIDTH;
 }
 
@@ -163,7 +161,7 @@ cover_button_clicked_cb (GtkWidget     *button,
 /* -- drag and drop -- */
 
 
-void
+static void
 cover_button_drag_data_received  (GtkWidget          *widget,
 				  GdkDragContext     *context,
 				  gint                x,
@@ -325,6 +323,7 @@ goo_player_info_finalize (GObject *object)
 	info = GOO_PLAYER_INFO (object);
 	g_free (info->priv->total_time);
 	if (info->priv->update_id != 0) {
+		g_object_unref (info->priv->original_cover);
 		g_source_remove (info->priv->update_id);
 		info->priv->update_id = 0;
 	}
@@ -333,28 +332,6 @@ goo_player_info_finalize (GObject *object)
 }
 
 
-static void
-update_subtitle (GooPlayerInfo *info,
-		 TrackInfo     *track)
-{
- 	AlbumInfo *album;
-
-	album = goo_window_get_album (info->priv->window);
-
-	if ((album->title == NULL) || (album->artist == NULL)) {
-		g_free (info->priv->total_time);
-		info->priv->total_time = (track->length > 0) ? _g_format_duration_for_display (track->length * 1000) : NULL;
-		set_title2 (info, info->priv->total_time);
-	}
-	else {
-		set_title2 (info, album->artist);
-		set_title3 (info, album->title);
-		gtk_label_set_selectable (GTK_LABEL (info->priv->title2_label), TRUE);
-		gtk_label_set_selectable (GTK_LABEL (info->priv->title3_label), TRUE);
-	}
-}
-
-
 static void
 show_simple_text (GooPlayerInfo *info,
 		  const char     *text)
@@ -530,11 +507,11 @@ static void
 goo_player_info_set_cover (GooPlayerInfo *info,
 			   const char    *cover)
 {
-	gboolean cover_set = FALSE;
-
 	if (cover == NULL)
 		return;
 
+	g_clear_object (&info->priv->original_cover);
+
 	if (strcmp (cover, "no-disc") == 0) {
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (info->priv->notebook), 0);
 		gtk_image_set_from_stock (GTK_IMAGE (info->priv->status_image),
@@ -554,13 +531,14 @@ goo_player_info_set_cover (GooPlayerInfo *info,
 					  GTK_ICON_SIZE_DIALOG);
 	}
 	else {
-		GdkPixbuf *image;
+		info->priv->original_cover = gdk_pixbuf_new_from_file (cover, NULL);
+		if (info->priv->original_cover != NULL) {
+			GdkPixbuf *image;
 
-		image = gdk_pixbuf_new_from_file_at_size (cover, 80, 80, NULL);
-		if (image != NULL) {
+			image = gdk_pixbuf_scale_simple (info->priv->original_cover, COVER_SIZE, COVER_SIZE, GDK_INTERP_BILINEAR);
 			gtk_notebook_set_current_page (GTK_NOTEBOOK (info->priv->notebook), 1);
 			gtk_image_set_from_pixbuf (GTK_IMAGE (info->priv->cover_image), image);
-			cover_set = TRUE;
+
 			g_object_unref (image);
 		}
 		else
@@ -673,8 +651,5 @@ goo_player_info_new (GooWindow *window)
 GdkPixbuf *
 goo_player_info_get_cover (GooPlayerInfo *info)
 {
-	if (gtk_image_get_storage_type (GTK_IMAGE (info->priv->cover_image)) == GTK_IMAGE_PIXBUF)
-		return gtk_image_get_pixbuf (GTK_IMAGE (info->priv->cover_image));
-	else
-		return NULL;
+	return info->priv->original_cover;
 }
