@@ -434,8 +434,10 @@ goo_window_update_titles (GooWindow *window)
 				    -1);
 
 		/* Update the current track info. */
+
 		if ((window->priv->current_track != NULL)
-		    && (new_track->number == window->priv->current_track->number)) {
+		    && (new_track->number == window->priv->current_track->number))
+		{
 			track_info_unref (window->priv->current_track);
 			track_info_ref (new_track);
 			window->priv->current_track = new_track;
@@ -624,12 +626,14 @@ add_columns (GooWindow   *window,
 static void
 print_playlist (GooWindow *window)
 {
+#ifdef DEBUG
 	GList *scan;
 
 	debug (DEBUG_INFO, "PLAYLIST: ");
 	for (scan = window->priv->playlist; scan; scan = scan->next)
 		debug (DEBUG_INFO, "%d, ", GPOINTER_TO_INT (scan->data));
 	debug (DEBUG_INFO, "\n");
+#endif
 }
 
 
@@ -688,6 +692,54 @@ get_position_from_track_number (GooWindow   *window,
 }
 
 
+static void
+add_selected_track (GtkTreeModel *model,
+		    GtkTreePath  *path,
+		    GtkTreeIter  *iter,
+		    gpointer      data)
+{
+	GList    **list = data;
+	TrackInfo  *track;
+
+	gtk_tree_model_get (model, iter, COLUMN_TRACK_INFO, &track, -1);
+	*list = g_list_prepend (*list, track);
+}
+
+
+static gboolean
+set_selected_track_if_unique (GooWindow *window,
+			      int       *n_track)
+{
+	GtkTreeSelection *list_selection;
+	GList            *tracks;
+	gboolean          result;
+
+	if (window->priv->album->tracks == NULL)
+		return FALSE;
+
+	list_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (window->priv->list_view));
+	if (list_selection == NULL)
+		return FALSE;
+
+	tracks = NULL;
+	gtk_tree_selection_selected_foreach (list_selection, add_selected_track, &tracks);
+
+	if ((tracks != NULL) && (tracks->next == NULL)) {
+		TrackInfo *track;
+
+		track = tracks->data;
+		*n_track = track->number;
+		result = TRUE;
+	}
+	else
+		result = FALSE;
+
+	track_list_free (tracks);
+
+	return result;
+}
+
+
 static void
 create_playlist (GooWindow *window,
 		 gboolean   play_all,
@@ -695,7 +747,8 @@ create_playlist (GooWindow *window,
 {
 
 	GList *playlist;
-	int    pos = 0, i;
+	int    selected_track_number;
+	int    pos;
 
 	debug (DEBUG_INFO, "PLAY ALL: %d\n", play_all);
 	debug (DEBUG_INFO, "SHUFFLE: %d\n", shuffle);
@@ -704,57 +757,121 @@ create_playlist (GooWindow *window,
 		g_list_free (window->priv->playlist);
 	window->priv->playlist = NULL;
 
-	if (!play_all)
-		return;
-
 	playlist = NULL;
 
-	if (window->priv->current_track != NULL)
+	if (window->priv->current_track != NULL) {
+
+		/* ignore the currently playing track */
+
+		selected_track_number = window->priv->current_track->number;
 		pos = get_position_from_track_number (window, window->priv->current_track->number);
+	}
+	else if (set_selected_track_if_unique (window, &selected_track_number)) {
 
-	for (i = 0; i < window->priv->album->n_tracks; i++, pos = (pos + 1) % window->priv->album->n_tracks) {
-		int track_number;
+		/* put the selected track first */
 
-		track_number = get_track_number_from_position (window, pos);
-		if ((window->priv->current_track != NULL)
-		    && (window->priv->current_track->number == track_number))
-			continue;
-		playlist = g_list_prepend (playlist, GINT_TO_POINTER (track_number));
+		playlist = g_list_prepend (playlist, GINT_TO_POINTER (selected_track_number));
+		pos = get_position_from_track_number (window, selected_track_number);
+	}
+	else {
+		selected_track_number = -1;
+		pos = 0;
 	}
 
-	playlist = g_list_reverse (playlist);
+	if (play_all) {
+		int i;
 
-	if (shuffle) {
-		GRand *grand = g_rand_new ();
-		GList *random_list = NULL;
-		int    len = g_list_length (playlist);
+		for (i = 0;
+		     i < window->priv->album->n_tracks;
+		     i++, pos = (pos + 1) % window->priv->album->n_tracks)
+		{
+			int track_number;
 
-		while (playlist != NULL) {
-			GList *item;
+			track_number = get_track_number_from_position (window, pos);
+			if (track_number == selected_track_number)
+				/* skip the selected track */
+				continue;
 
-			pos = g_rand_int_range (grand, 0, len--);
-			item = g_list_nth (playlist, pos);
-			playlist = g_list_remove_link (playlist, item);
-			random_list = g_list_concat (random_list, item);
+			playlist = g_list_prepend (playlist, GINT_TO_POINTER (track_number));
 		}
-		playlist = random_list;
 
-		g_rand_free (grand);
+		playlist = g_list_reverse (playlist);
+
+		if (shuffle) {
+			GRand *grand = g_rand_new ();
+			GList *random_list = NULL;
+			int    len = g_list_length (playlist);
+
+			while (playlist != NULL) {
+				GList *item;
+
+				pos = g_rand_int_range (grand, 0, len--);
+				item = g_list_nth (playlist, pos);
+				playlist = g_list_remove_link (playlist, item);
+				random_list = g_list_concat (random_list, item);
+			}
+			playlist = random_list;
+
+			g_rand_free (grand);
+		}
 	}
 
 	window->priv->playlist = playlist;
+
+	print_playlist (window);
+}
+
+
+static void
+update_playlist_after_started_next (GooWindow *window)
+{
+	GList *current;
+
+	if (window->priv->playlist == NULL)
+		return;
+
+	current = window->priv->playlist;
+	window->priv->playlist = g_list_remove_link (window->priv->playlist, current);
+
+	g_list_free (current);
+}
+
+
+static void
+update_next_track_to_play (GooWindow *window)
+{
+	GList *track;
+	int    next_track_to_play;
+
+	for (track = window->priv->playlist; track; track = track->next)
+		if (GPOINTER_TO_INT (track->data) == goo_player_get_current_track (window->priv->player))
+			break;
+
+	if (track != NULL) {
+		if (track->next != NULL)
+			next_track_to_play = GPOINTER_TO_INT (track->next->data);
+		else
+			next_track_to_play = -1;
+	}
+	else if (window->priv->playlist != NULL)
+		next_track_to_play = GPOINTER_TO_INT (window->priv->playlist->data);
+	else
+		next_track_to_play = -1;
+
+	goo_player_set_next_track (window->priv->player, next_track_to_play);
 }
 
 
 static void
 play_track (GooWindow *window,
-	    int        track_number)
+	    int        track_to_play)
 {
-	goo_player_seek_track (window->priv->player, track_number);
+	goo_player_seek_track (window->priv->player, track_to_play);
+	update_next_track_to_play (window);
 }
 
 
-static void
+static gboolean
 play_next_track_in_playlist (GooWindow *window)
 {
 	gboolean  play_all;
@@ -775,16 +892,19 @@ play_next_track_in_playlist (GooWindow *window)
 		next = window->priv->playlist;
 	}
 
-	print_playlist (window);
+	if (next != NULL) {
+		int pos;
 
-	if (next == NULL)
-		goo_window_stop (window);
-	else {
-		int pos = GPOINTER_TO_INT (next->data);
-		play_track (window, pos);
+		pos = GPOINTER_TO_INT (next->data);
 		window->priv->playlist = g_list_remove_link (window->priv->playlist, next);
 		g_list_free (next);
+
+		play_track (window, pos);
 	}
+	else
+		goo_window_stop (window);
+
+	return (next != NULL);
 }
 
 
@@ -802,6 +922,7 @@ pref_playlist_playall_changed (GSettings  *settings,
 	play_all = g_settings_get_boolean (window->priv->settings_playlist, PREF_PLAYLIST_PLAYALL);
 	shuffle = g_settings_get_boolean (window->priv->settings_playlist, PREF_PLAYLIST_SHUFFLE);
 	create_playlist (window, play_all, shuffle);
+	update_next_track_to_play (window);
 
 	window_update_sensitivity (window);
 }
@@ -821,6 +942,7 @@ pref_playlist_shuffle_changed (GSettings  *settings,
 	play_all = g_settings_get_boolean (window->priv->settings_playlist, PREF_PLAYLIST_PLAYALL);
 	shuffle = g_settings_get_boolean (window->priv->settings_playlist, PREF_PLAYLIST_SHUFFLE);
 	create_playlist (window, play_all, shuffle);
+	update_next_track_to_play (window);
 }
 
 
@@ -839,7 +961,6 @@ static void
 goo_window_unrealize (GtkWidget *widget)
 {
 	GooWindow *window;
-	gboolean   playlist_visible;
 
 	window = GOO_WINDOW (widget);
 
@@ -1125,6 +1246,9 @@ get_action_name (GooPlayerAction action)
 	case GOO_PLAYER_ACTION_METADATA:
 		name = "METADATA";
 		break;
+	case GOO_PLAYER_ACTION_STARTED_NEXT:
+		name = "STARTED_NEXT";
+		break;
 	default:
 		name = "???";
 		break;
@@ -1566,6 +1690,12 @@ player_done_cb (GooPlayer       *player,
 		notify_current_state (window, action);
 		break;
 
+	case GOO_PLAYER_ACTION_STARTED_NEXT:
+		notify_current_state (window, GOO_PLAYER_ACTION_PLAY);
+		update_playlist_after_started_next (window);
+		update_next_track_to_play (window);
+		break;
+
 	default:
 		break;
 	}
@@ -2250,19 +2380,13 @@ goo_window_play (GooWindow *window)
 void
 goo_window_play_selected (GooWindow *window)
 {
-	GList *tracks;
-
-	tracks = goo_window_get_tracks (window, TRUE);
-
-	if (g_list_length (tracks) == 1) {
-		TrackInfo *track = tracks->data;
+	int track_n;
 
+	if (set_selected_track_if_unique (window, &track_n)) {
 		goo_window_stop (window);
-		goo_window_set_current_track (window, track->number);
+		goo_window_set_current_track (window, track_n);
 		goo_window_play (window);
 	}
-
-	track_list_free (tracks);
 }
 
 
@@ -2305,29 +2429,24 @@ goo_window_next (GooWindow *window)
 	if (! goo_player_is_audio_cd (window->priv->player))
 		return;
 
+	if (play_next_track_in_playlist (window))
+		return;
+
 	if (window->priv->current_track != NULL) {
-		if (window->priv->playlist == NULL) {
-			int current_track = window->priv->current_track->number;
-			int current_pos;
-			int new_pos, new_track;
+		int current_track = window->priv->current_track->number;
+		int current_pos;
+		int new_pos, new_track;
 
-			goo_window_stop (window);
-
-			current_pos = get_position_from_track_number (window, current_track);
-			new_pos = MIN (current_pos + 1, window->priv->album->n_tracks - 1);
-			new_track = get_track_number_from_position (window, new_pos);
-			goo_window_set_current_track (window, new_track);
-
-			goo_window_play (window);
-		}
-		else
-			play_next_track_in_playlist (window);
-	}
-	else {
 		goo_window_stop (window);
-		goo_window_set_current_track (window, 0);
+
+		current_pos = get_position_from_track_number (window, current_track);
+		new_pos = MIN (current_pos + 1, window->priv->album->n_tracks - 1);
+		new_track = get_track_number_from_position (window, new_pos);
+		goo_window_set_current_track (window, new_track);
 		goo_window_play (window);
 	}
+	else
+		goo_window_play (window);
 }
 
 
@@ -2342,26 +2461,20 @@ goo_window_prev (GooWindow *window)
 	if (window->priv->album->n_tracks == 0)
 		return;
 
-	goo_window_stop (window);
-
 	if (window->priv->current_track != NULL) {
 		int current_track, current_pos;
 
+		goo_window_stop (window);
+
 		current_track = window->priv->current_track->number;
 		current_pos = get_position_from_track_number (window, current_track);
-
-		/* FIXME
-		if (goo_player_info_get_progress (GOO_PLAYER_INFO (window->priv->info)) * window->priv->current_track->length > 4)
-			new_pos = current_pos;
-		else
-		*/
-			new_pos = MAX (current_pos - 1, 0);
+		new_pos = MAX (current_pos - 1, 0);
+		goo_window_set_current_track (window, get_track_number_from_position (window, new_pos));
+		goo_window_play (window);
 	}
 	else
-		new_pos = window->priv->album->n_tracks - 1;
+		goo_window_play (window);
 
-	goo_window_set_current_track (window, get_track_number_from_position (window, new_pos));
-	goo_window_play (window);
 }
 
 
@@ -2382,20 +2495,6 @@ goo_window_set_drive (GooWindow    *window,
 }
 
 
-static void
-add_selected_track (GtkTreeModel *model,
-		   GtkTreePath  *path,
-		   GtkTreeIter  *iter,
-		   gpointer      data)
-{
-	GList    **list = data;
-	TrackInfo  *track;
-
-	gtk_tree_model_get (model, iter, COLUMN_TRACK_INFO, &track, -1);
-	*list = g_list_prepend (*list, track);
-}
-
-
 AlbumInfo *
 goo_window_get_album (GooWindow *window)
 {
@@ -2443,16 +2542,23 @@ goo_window_get_player_info (GooWindow *window)
 }
 
 
-void
+#define MIN_COVER_SIZE 10
+
+
+gboolean
 goo_window_set_cover_image_from_pixbuf (GooWindow *window,
 					GdkPixbuf *image)
 {
 	GError    *error = NULL;
 	GdkPixbuf *frame;
 	char      *cover_filename;
+	gboolean   image_saved;
 
 	if (image == NULL)
-		return;
+		return FALSE;
+
+	if ((gdk_pixbuf_get_width (image) < MIN_COVER_SIZE) || (gdk_pixbuf_get_height (image) < MIN_COVER_SIZE))
+		return FALSE;
 
 	frame = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (image),
 				gdk_pixbuf_get_has_alpha (image),
@@ -2470,44 +2576,51 @@ goo_window_set_cover_image_from_pixbuf (GooWindow *window,
 	cover_filename = goo_window_get_cover_filename (window);
 	debug (DEBUG_INFO, "SAVE IMAGE %s\n", cover_filename);
 
-	if (! gdk_pixbuf_save (frame, cover_filename, "png", &error, NULL))
-		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (window),
-						   _("Could not save cover image"),
-						   &error);
-	else {
+	image_saved = gdk_pixbuf_save (frame, cover_filename, "png", &error, NULL);
+	if (image_saved) {
 		goo_window_set_current_cd_autofetch (window, FALSE);
 		goo_window_update_cover (window);
 	}
+	else
+		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (window),
+						   _("Could not save cover image"),
+						   &error);
 
 	g_free (cover_filename);
 	g_object_unref (frame);
+
+	return image_saved;
 }
 
 
-void
+gboolean
 goo_window_set_cover_image (GooWindow  *window,
 			    const char *filename)
 {
 	GdkPixbuf *image;
 	GError    *error = NULL;
+	gboolean   result;
 
 	if (window->priv->hibernate)
-		return;
+		return FALSE;
 
 	image = gdk_pixbuf_new_from_file (filename, &error);
 	if (image == NULL) {
 		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (window),
 						   _("Could not load image"),
 						   &error);
-		return;
+		return FALSE;
 	}
-	goo_window_set_cover_image_from_pixbuf (window, image);
+
+	result = goo_window_set_cover_image_from_pixbuf (window, image);
 
 	g_object_unref (image);
+
+	return result;
 }
 
 
-void
+gboolean
 goo_window_set_cover_image_from_data (GooWindow *window,
 				      void      *buffer,
 				      gsize      count)
@@ -2515,24 +2628,28 @@ goo_window_set_cover_image_from_data (GooWindow *window,
 	GInputStream *stream;
 	GdkPixbuf    *image;
 	GError       *error = NULL;
+	gboolean      result;
 
 	if (window->priv->hibernate)
-		return;
+		return FALSE;
 
 	stream = g_memory_input_stream_new_from_data (buffer, count, NULL);
 	image = gdk_pixbuf_new_from_stream (stream, NULL, &error);
+
 	if (image == NULL) {
 		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (window),
 						   _("Could not load image"),
 						   &error);
 		g_object_unref (stream);
-		return;
+		return FALSE;
 	}
 
-	goo_window_set_cover_image_from_pixbuf (window, image);
+	result = goo_window_set_cover_image_from_pixbuf (window, image);
 
 	g_object_unref (image);
 	g_object_unref (stream);
+
+	return result;
 }
 
 
